/* This file is in the public domain */

#include <string>
#include <cstdlib>
#include <ctime>

#include <opencl/filters.h>

#include <opencl/square.h>
#include <opencl/randpool.h>
#include <opencl/x917.h>

#include <opencl/ent_file.h>

#if defined(OPENCL_EXT_ENTROPY_SRC_UNIX_POPEN)
#include <opencl/unix_rnd.h>
#endif

using namespace OpenCL;

/*************************************************/
/*
   This is the global RNG used in various spots. Seems to make most sense to
   declare it here.
*/

// this one is useful for debugging
class RNG : public OpenCL::RandomNumberGenerator
   {
   public:
      RNG() : RandomNumberGenerator("fake rng")
         {
         int x = time(0);
         std::srand(x);
         }
      byte random() { return (std::rand() % 255); }
      void clear() throw() {}
      void add_entropy(const byte[], u32bit) throw() {}
      void add_entropy(EntropySource&, bool) {}
   };

OpenCL::X917<OpenCL::Square> local_rng;
//RNG local_rng;

OpenCL::RandomNumberGenerator& rng = local_rng;
/*************************************************/

/* Not too useful generally; just dumps random bits */
template<typename R>
class RNG_Filter : public Filter
   {
   public:
      void write(const byte[], u32bit);
      RNG_Filter() : buffer(1024) {}
   private:
      R rng;
      SecureVector<byte> buffer;
      u32bit position;
   };

template<typename B>
void RNG_Filter<B>::write(const byte input[], u32bit length)
   {
   buffer.copy(position, input, length);
   if(position + length >= buffer.size())
      {
      rng.randomize(buffer, buffer.size());
      send(buffer, buffer.size());
      input += (buffer.size() - position);
      length -= (buffer.size() - position);
      while(length >= buffer.size())
         {
         /* This actually totally ignores the input, but it doesn't matter,
            because this is only for benchmark purposes and we just want to
            test speed. Anyway, if the RNG is good you can't tell the diff */
         rng.randomize(buffer, buffer.size());
         send(buffer, buffer.size());
         input += buffer.size();
         length -= buffer.size();
         }
       buffer.copy(input, length);
      position = 0;
      }
   position += length;
   }

/* A wrappr class to convert an EntropySource into a pseudo-RNG */
template<typename E>
class ES_TO_RNG
   {
   public:
      void randomize(byte buf[], u32bit size)
         {
         u32bit need = size;
         while(need)
            need -= es.slow_poll(buf + size - need, need);
         }

   private:
      E es;
   };

Filter* lookup_rng(const std::string& algname)
   {
   if(algname == "X917<Square>")
      return new RNG_Filter< X917<Square> >;
   else if(algname == "Randpool")
      return new RNG_Filter<Randpool>;

   else if(algname == "File_EntropySource")
      return new RNG_Filter< ES_TO_RNG<File_EntropySource> >;

#if defined(OPENCL_EXT_ENTROPY_SRC_UNIX_POPEN)
   else if(algname == "Unix_EntropySource")
      return new RNG_Filter< ES_TO_RNG<Unix_EntropySource> >;
#endif
   else return 0;
   }
